@@ -31,8 +31,8 @@ import (
 	"golang.org/x/text/language"
 
 	api "github.com/osrg/gobgp/v3/api"
-	"github.com/osrg/gobgp/v3/internal/pkg/config"
 	"github.com/osrg/gobgp/v3/pkg/apiutil"
+	"github.com/osrg/gobgp/v3/pkg/config/oc"
 	"github.com/osrg/gobgp/v3/pkg/packet/bgp"
 )
 
@@ -96,7 +96,7 @@ func getNeighbors(address string, enableAdv bool) ([]*api.Peer, error) {
 	return l, err
 }
 
-func getASN(p *api.Peer) string {
+func getRemoteASN(p *api.Peer) string {
 	asn := "*"
 	if p.State.PeerAsn > 0 {
 		asn = fmt.Sprint(p.State.PeerAsn)
@@ -104,6 +104,14 @@ func getASN(p *api.Peer) string {
 	return asn
 }
 
+func getLocalASN(p *api.Peer) string {
+	asn := "*"
+	if p.State.LocalAsn > 0 {
+		asn = fmt.Sprint(p.State.LocalAsn)
+	}
+	return asn
+}
+
 func counter(p *api.Peer) (uint64, uint64, uint64, error) {
 	accepted := uint64(0)
 	received := uint64(0)
@@ -181,7 +189,7 @@ func showNeighbors(vrf string) error {
 		} else if j := len(n.State.NeighborAddress); j > maxaddrlen {
 			maxaddrlen = j
 		}
-		if l := len(getASN(n)); l > maxaslen {
+		if l := len(getRemoteASN(n)); l > maxaslen {
 			maxaslen = l
 		}
 		timeStr := "never"
@@ -236,7 +244,7 @@ func showNeighbors(vrf string) error {
 			neigh = n.Conf.NeighborInterface
 		}
 		received, accepted, _, _ := counter(n)
-		fmt.Printf(format, neigh, getASN(n), timedelta[i], formatFsm(n.State.AdminState, n.State.SessionState), fmt.Sprint(received), fmt.Sprint(accepted))
+		fmt.Printf(format, neigh, getRemoteASN(n), timedelta[i], formatFsm(n.State.AdminState, n.State.SessionState), fmt.Sprint(received), fmt.Sprint(accepted))
 	}
 
 	return nil
@@ -255,7 +263,7 @@ func showNeighbor(args []string) error {
 		return nil
 	}
 
-	fmt.Printf("BGP neighbor is %s, remote AS %s", p.State.NeighborAddress, getASN(p))
+	fmt.Printf("BGP neighbor is %s, remote AS %s", p.State.NeighborAddress, getRemoteASN(p))
 
 	if p.RouteReflector.RouteReflectorClient {
 		fmt.Printf(", route-reflector-client\n")
@@ -277,6 +285,7 @@ func showNeighbor(args []string) error {
 		fmt.Print("\n")
 	}
 	fmt.Printf("  BGP OutQ = %d, Flops = %d\n", p.State.Queues.Output, p.State.Flops)
+	fmt.Printf("  Local address is %s, local ASN: %s\n", p.Transport.LocalAddress, getLocalASN(p))
 	fmt.Printf("  Hold time is %d, keepalive interval is %d seconds\n", int(p.Timers.State.NegotiatedHoldTime), int(p.Timers.State.KeepaliveInterval))
 	fmt.Printf("  Configured hold time is %d, keepalive interval is %d seconds\n", int(p.Timers.Config.HoldTime), int(p.Timers.Config.KeepaliveInterval))
 
@@ -474,6 +483,16 @@ func showNeighbor(args []string) error {
 				fmt.Println("      Remote:")
 				fmt.Printf("         name: %s, domain: %s\n", m.(*bgp.CapFQDN).HostName, m.(*bgp.CapFQDN).DomainName)
 			}
+		case bgp.BGP_CAP_SOFT_VERSION:
+			fmt.Printf("    %s:\t%s\n", c.Code(), support)
+			if m := lookup(c, lcaps); m != nil {
+				fmt.Println("      Local:")
+				fmt.Printf("         %s\n", m.(*bgp.CapSoftwareVersion).SoftwareVersion)
+			}
+			if m := lookup(c, rcaps); m != nil {
+				fmt.Println("      Remote:")
+				fmt.Printf("         %s\n", m.(*bgp.CapSoftwareVersion).SoftwareVersion)
+			}
 		default:
 			fmt.Printf("    %s:\t%s\n", c.Code(), support)
 		}
@@ -822,6 +841,7 @@ func showNeighborRib(r string, name string, args []string) error {
 	showMUP := false
 	showIdentifier := bgp.BGP_ADD_PATH_NONE
 	validationTarget := ""
+	rd := ""
 
 	def := addr2AddressFamily(net.ParseIP(name))
 	switch r {
@@ -844,8 +864,6 @@ func showNeighborRib(r string, name string, args []string) error {
 	switch rf {
 	case bgp.RF_IPv4_MPLS, bgp.RF_IPv6_MPLS, bgp.RF_IPv4_VPN, bgp.RF_IPv6_VPN, bgp.RF_EVPN:
 		showLabel = true
-	case bgp.RF_MUP_IPv4, bgp.RF_MUP_IPv6:
-		showMUP = true
 	}
 
 	var filter []*api.TableLookupPrefix
@@ -855,7 +873,11 @@ func showNeighborRib(r string, name string, args []string) error {
 		case bgp.RF_EVPN:
 			// Uses target as EVPN Route Type string
 		case bgp.RF_MUP_IPv4, bgp.RF_MUP_IPv6:
-			// Uses target as MUP Route Type string
+			// Uses target as MUP Route Type string or route key
+			// Only t1st has MUP specific columns
+			if target == "t1st" {
+				showMUP = true
+			}
 		default:
 			if _, _, err = parseCIDRorIP(target); err != nil {
 				return err
@@ -868,6 +890,25 @@ func showNeighborRib(r string, name string, args []string) error {
 				option = api.TableLookupPrefix_LONGER
 			} else if args[0] == "shorter-prefixes" {
 				option = api.TableLookupPrefix_SHORTER
+			} else if args[0] == "rd" {
+				switch rf {
+				case bgp.RF_IPv4_VPN, bgp.RF_IPv6_VPN:
+				case bgp.RF_EVPN:
+					return fmt.Errorf("route distinguisher option for %q family is not implemented yet", rf)
+				default:
+					return fmt.Errorf("route distinguisher is not applicable to %q family", rf)
+				}
+
+				if len(args) < 2 {
+					return fmt.Errorf("no route distinguisher value specified")
+				}
+				args = args[1:]
+
+				rd = args[0]
+				_, err = bgp.ParseRouteDistinguisher(rd)
+				if err != nil {
+					return err
+				}
 			} else if args[0] == "validation" {
 				if r != cmdAdjIn {
 					return fmt.Errorf("RPKI information is supported for only adj-in")
@@ -880,6 +921,7 @@ func showNeighborRib(r string, name string, args []string) error {
 		}
 		filter = []*api.TableLookupPrefix{{
 			Prefix: target,
+			Rd:     rd,
 			Type:   option,
 		},
 		}
@@ -1241,7 +1283,7 @@ func modNeighbor(cmdType string, args []string) error {
 
 	getNeighborAddress := func() (string, error) {
 		if unnumbered {
-			return config.GetIPv6LinkLocalNeighborAddress(m["interface"][0])
+			return oc.GetIPv6LinkLocalNeighborAddress(m["interface"][0])
 		}
 		return m[""][0], nil
 	}
