@@ -1294,47 +1294,69 @@ func Test_MpReachNLRIAttribute_MUPDirectSegmentDiscoveryRoute(t *testing.T) {
 
 func Test_MpReachNLRIAttribute_MUPType1SessionTransformedRoute(t *testing.T) {
 	assert := assert.New(t)
-
-	nlris := make([]*apb.Any, 0, 1)
 	rd, err := apb.New(&api.RouteDistinguisherTwoOctetASN{
 		Admin:    65000,
 		Assigned: 100,
 	})
 	assert.Nil(err)
-	a, err := apb.New(&api.MUPType1SessionTransformedRoute{
-		Rd:                    rd,
-		Prefix:                "192.168.100.1/32",
-		Teid:                  12345,
-		Qfi:                   9,
-		EndpointAddressLength: 32,
-		EndpointAddress:       "10.0.0.1",
-	})
-	assert.Nil(err)
-	nlris = append(nlris, a)
-
-	input := &api.MpReachNLRIAttribute{
-		Family: &api.Family{
-			Afi:  api.Family_AFI_IP,
-			Safi: api.Family_SAFI_MUP,
+	tests := []struct {
+		name string
+		in   *api.MUPType1SessionTransformedRoute
+	}{
+		{
+			name: "IPv4",
+			in: &api.MUPType1SessionTransformedRoute{
+				Rd:                    rd,
+				Prefix:                "192.168.100.1/32",
+				Teid:                  12345,
+				Qfi:                   9,
+				EndpointAddressLength: 32,
+				EndpointAddress:       "10.0.0.1",
+			},
+		},
+		{
+			name: "IPv4_with_SourceAddress",
+			in: &api.MUPType1SessionTransformedRoute{
+				Rd:                    rd,
+				Prefix:                "192.168.100.1/32",
+				Teid:                  12345,
+				Qfi:                   9,
+				EndpointAddressLength: 32,
+				EndpointAddress:       "10.0.0.1",
+				SourceAddressLength:   32,
+				SourceAddress:         "10.0.0.2",
+			},
 		},
-		NextHops: []string{"0.0.0.0"},
-		Nlris:    nlris,
 	}
+	for _, tt := range tests {
+		a, err := apb.New(tt.in)
+		assert.Nil(err)
+		nlris := []*apb.Any{a}
 
-	a, err = apb.New(input)
-	assert.Nil(err)
-	n, err := UnmarshalAttribute(a)
-	assert.Nil(err)
+		input := &api.MpReachNLRIAttribute{
+			Family: &api.Family{
+				Afi:  api.Family_AFI_IP,
+				Safi: api.Family_SAFI_MUP,
+			},
+			NextHops: []string{"0.0.0.0"},
+			Nlris:    nlris,
+		}
 
-	output, _ := NewMpReachNLRIAttributeFromNative(n.(*bgp.PathAttributeMpReachNLRI))
-	assert.Equal(input.Family.Afi, output.Family.Afi)
-	assert.Equal(input.Family.Safi, output.Family.Safi)
-	assert.Equal(input.NextHops, output.NextHops)
-	assert.Equal(1, len(output.Nlris))
-	for idx, inputNLRI := range input.Nlris {
-		outputNLRI := output.Nlris[idx]
-		assert.Equal(inputNLRI.TypeUrl, outputNLRI.TypeUrl)
-		assert.Equal(inputNLRI.Value, outputNLRI.Value)
+		a, err = apb.New(input)
+		assert.Nil(err)
+		n, err := UnmarshalAttribute(a)
+		assert.Nil(err)
+
+		output, _ := NewMpReachNLRIAttributeFromNative(n.(*bgp.PathAttributeMpReachNLRI))
+		assert.Equal(input.Family.Afi, output.Family.Afi)
+		assert.Equal(input.Family.Safi, output.Family.Safi)
+		assert.Equal(input.NextHops, output.NextHops)
+		assert.Equal(1, len(output.Nlris))
+		for idx, inputNLRI := range input.Nlris {
+			outputNLRI := output.Nlris[idx]
+			assert.Equal(inputNLRI.TypeUrl, outputNLRI.TypeUrl)
+			assert.Equal(inputNLRI.Value, outputNLRI.Value)
+		}
 	}
 }
 
