@@ -17,6 +17,9 @@ import sys
 import time
 import unittest
 
+import collections
+collections.Callable = collections.abc.Callable
+
 import nose
 
 from lib import base
